
#TODO
# - jajka gina z glodu!!
# - game over when all is dead
# - zbieranie jajek
# - kupowanie ziarna
# - latajacy lis po ustrzeleniu
# - efekty przy strzelaniu
# - liczenie dorastajacych kur


# std lib
import math
from random import random, randint

# pyglet
import pyglet
from pyglet.gl import *
from pyglet.window.key import *

# cocos
import cocos
from cocos.director import director
from cocos.sprite import Sprite
from cocos import euclid
from cocos.actions.interval_actions import MoveBy, MoveTo, ScaleBy, JumpBy, RotateBy, Delay, Accelerate
from cocos.actions.instant_actions import Hide, Show, Place, CallFunc
from cocos.actions.base_actions import Reverse, Repeat
from cocos.text import Label
from cocos.scenes.transitions import FadeTransition

import soundex
#import gameover
from constants import *
import hunting

EGG_AGE = (0, 9)
CHICK_AGE = (10, 29)
CHICKEN_AGE = (30, 39)
ADULT_AGE = (40, 69)
OLD_AGE = (70, 100)

def is_egg(a):
    if a.age >= EGG_AGE[0] and a.age <= EGG_AGE[1]:
        return True
    return False

def is_chick(a):
    if a.age >= CHICK_AGE[0] and a.age <= CHICK_AGE[1]:
        return True
    return False

def is_chicken(a):
    if a.age >= CHICKEN_AGE[0] and a.age <= CHICKEN_AGE[1]:
        return True
    return False

def is_hen(a):
    if a.age >= ADULT_AGE[0] and a.age <= ADULT_AGE[1] and a.male == False:
        return True
    return False

def is_rooster(a):
    if a.age >= ADULT_AGE[0] and a.age <= ADULT_AGE[1] and a.male == True:
        return True
    return False

DAY_LENGTH = 12

def log(txt):
    print txt

class BackgroundLayer(cocos.layer.Layer):
    def __init__(self):
        super(BackgroundLayer, self).__init__()
        grass = Sprite('grass_1_4.png', position=(1024/2, 768/2))
        self.add(grass)

class Animal(object):
    def __init__(self, x, y, direction, male, age=0):
        self.age = age
        self.male = male
        self._calculate_step()

        scale = self._calculate_scale(y)
        self._set_sprites(x, y, scale)
        if direction == 0:
            self.direction = -1
            self.sprite_l.schedule_interval(self._move_chicken, 2)
        else:
            self.direction = 1
            self.sprite_r.schedule_interval(self._move_chicken, 2)

    def _calculate_scale(self, y):
        return (1024-y+500)/float(1024-80+500)

    def _set_sprites(self, x, y, scale):
        img = self._get_image_name()
        self.head = None
        if self.age >= ADULT_AGE[0] and not self.male: # hen
            body = Sprite('hen_body_r.png', position=(x, y), scale=scale)
            head = Sprite('hen_head_r.png', position=(2, 18), anchor=(0, 30), scale=scale)
            self.head = head
            body.add(head)
            self.sprite_r = body
            self.sprite_l = Sprite(img+'.png', position=(x, y), scale=scale)
        else:
            self.sprite_r = Sprite(img+'_r.png', position=(x, y), scale=scale)
            self.sprite_l = Sprite(img+'.png', position=(x, y), scale=scale)

    def _get_image_name(self):
        if self.age <= CHICK_AGE[1]: # chick
            return "chick_3_1"
        elif self.age <= CHICKEN_AGE[1]: # chicken
            return "chicken_1"
        else: # hen/rooster
            if self.male:
                return "rooster_3_2"
            else:
                return "hen_3_3"
        assert False

    def _calculate_step(self):
        if self.age <= EGG_AGE[1]:   # egg
            self.step = 0
        elif self.age <= CHICK_AGE[1]: # chicken
            self.step = 10
        elif self.age <= CHICKEN_AGE[1]: # chicken
            self.step = 20
        elif self.age <= ADULT_AGE[1]: # hen/rooster
            if self.male:
                self.step = 50
            else:
                self.step = 40
        elif self.age <= OLD_AGE[1]: # very old
            self.step = 0
        else:                # dead
            self.step = 0

    def _do_bow(self):
        if self.head:
            r = RotateBy(70, 0.5)
            a = r + Reverse(r)
            self.head.do(a)

    def _move_chicken(self, dt):
        if self.age >= OLD_AGE[1]:
            return
        if self.direction == -1:
            sprite = self.sprite_l
            sprite_2 = self.sprite_r
        else:
            sprite = self.sprite_r
            sprite_2 = self.sprite_l
        if (sprite.x > 80 and self.direction == -1) or (sprite.x < 1024-80 and self.direction == 1):
            a = Accelerate(MoveBy((self.direction*random()*50, 0), 1), 2)
            if self.direction == 1:
                a = a + CallFunc(self._do_bow)
            sprite.do(a)
            sprite.unschedule(self._move_chicken)
            sprite.schedule_interval(self._move_chicken, 1+random()*2)
        else:
            self.direction = -1 * self.direction
            parent = sprite.parent
            sprite.unschedule(self._move_chicken)
            sprite_2.x = sprite.x
            sprite_2.y = sprite.y
            parent.remove(sprite)
            parent.add_animal(sprite_2)
            step_x = self.direction * (random()+0.5) * self.step
            sprite_2.do(Accelerate(MoveBy((step_x, 0), 1), 2))
            sprite_2.schedule_interval(self._move_chicken, 1+random()*2)


    def get_sprite(self):
        if self.direction == -1:
            return self.sprite_l
        else:
            return self.sprite_r

    def update(self, c):
        self.age += 1
        if self.age == CHICKEN_AGE[0]:
            sprite = self.get_sprite()
            parent = sprite.parent
            img = self._get_image_name()
            scale = self._calculate_scale(sprite.y)
            self.sprite_r = Sprite(img+'_r.png', position=(sprite.x, sprite.y), scale=scale)
            self.sprite_l = Sprite(img+'.png', position=(sprite.x, sprite.y), scale=scale)
            parent.remove(sprite)
            sprite = self.get_sprite()
            parent.add(sprite)
        elif self.age == OLD_AGE[1]:
            self.kill()

    def kill(self):
        if self.direction == -1:
            sprite = self.sprite_l
            r = RotateBy(180, 1)
        else:
            sprite = self.sprite_r
            r = RotateBy(-180, 1)
        for cb in sprite.scheduled_calls + sprite.scheduled_interval_calls:
            sprite.unschedule(cb)
        if sprite.are_actions_running():
            sprite.stop()
        a = JumpBy(height=100, duration=1) | r
        a = a + Delay(0.5) + Hide() + CallFunc(self.destroy)
        sprite.do(a)

    def destroy(self):
        sprite = self.get_sprite()
        sprite.stop()
        parent = sprite.parent
        parent.remove(sprite)
        for cb in sprite.scheduled_calls + sprite.scheduled_interval_calls:
            sprite.unschedule(cb)
        
class Counter(object):
    def __init__(self, layer, img, num, x, y, scale, price=0):
        self.lbl = Label(str(num), position=(x-10, y-50), bold=True, font_size=14, font_name=FONTS)
        layer.add(self.lbl)
        self.sprite = Sprite(img, position=(x, y), scale=scale)
        layer.add(self.sprite)
        self.cnt = num
        self.price = price

    def set(self, num):
        self.cnt = num
        self.lbl.element.text = str(self.cnt)

    def add(self, num):
        self.cnt += num
        self.lbl.element.text = str(self.cnt)

    def sell(self, animal, cash_cnt, dollars):
        if self.cnt == 0:
            return
        self.add(-1)
        cash_cnt.add(self.price)
        m = Place((animal.x, animal.y)) + Delay(0.5) + MoveTo((cash_cnt.sprite.x, cash_cnt.sprite.y), 0.6) 
        m = m + Delay(0.5) + Place((100, 900))
        m = Accelerate(m, 1.4)
        dollars.do(m)
        

def is_inside(sprite, x_, y_):
    x = sprite.x
    y = sprite.y
    w2 = sprite.width / 2
    h2 = sprite.height / 2
    #log("a %d %d is in %d %d - %d %d" % (x_, y_, x, y, w2, h2))
    if x_ > x - w2 and x_ < x + w2 and y_ > y - h2 and y_ < y + h2:
        return True
    return False

class Feeder(object):
    def __init__(self, farm, grain_cnt, x, y):
        self.farm = farm
        self.grain_cnt = grain_cnt
        self.feeder_btn = Sprite('feeder_1.png', position=(x, y))
        self.last_feeding_time = 0

    def is_inside(self, x, y):
        return is_inside(self.feeder_btn, x, y)

    def clicked(self):
        self.last_feeding_time = self.farm.clock
        if self.grain_cnt.cnt < 10:
            soundex.play("empty.mp3")
            return
        soundex.play("grains1.mp3")
        self.grain_cnt.add(-10)
        if randint(0, 1) == 0:
            d = 1
        else:
            d = -1
        m = MoveBy((d*20, 50), 0.3) + MoveBy((d*30, 50), 0.3) + MoveBy((d*40, 50), 0.3)
        m = Accelerate(m, 1.4)
        self.feeder_btn.do(m + Reverse(m))


class CtrlLayer(cocos.layer.Layer):
    is_event_handler = True
    def __init__(self):
        super(CtrlLayer, self).__init__()
        #director.interpreter_locals = {'hen': hen, 'bg': self}
        self.animals = self._generate_animals(10, ADULT_AGE, False) # hens
        self.animals.extend(self._generate_animals(5, ADULT_AGE, True)) # roosters
        self.animals.extend(self._generate_animals(7, CHICKEN_AGE, False)) # chickens - female
        self.animals.extend(self._generate_animals(3, CHICKEN_AGE, True)) # chickens - male
        self.animals.extend(self._generate_animals(7, CHICK_AGE, False)) # chicks - female
        self.animals.extend(self._generate_animals(3, CHICK_AGE, True)) # chicks - male

        # control panel
        self.hens_cnt = Counter(self, "hen_3_3.png", 0, 50, 60, 0.4, 5)
        self.add(self.hens_cnt.sprite)
        self.roosters_cnt = Counter(self, "rooster_3_2.png", 0, 120, 60, 0.3, 5)
        self.add(self.roosters_cnt.sprite)
        self.chickens_cnt = Counter(self, "chicken_1.png", 0, 180, 60, 0.4, 10)
        self.add(self.chickens_cnt.sprite)
        self.chicks_cnt = Counter(self, "chick_3_1.png", 0, 220, 60, 0.4)
        self.add(self.chicks_cnt.sprite)
        self.eggs_cnt = Counter(self, "egg.png", 0, 280, 60, 0.4, 3)
        self.add(self.eggs_cnt.sprite)
        self._count_animals()
        self.cash_cnt = Counter(self, "dollars_a.png", 0, 360, 60, 0.6)
        self.add(self.cash_cnt.sprite)
        self.grain_cnt = Counter(self, "grain.png", 80, 440, 60, 0.6, 1)
        self.add(self.grain_cnt.sprite)

        self.feeder = Feeder(self, self.grain_cnt, 515, 50)
        self.add(self.feeder.feeder_btn, z=10000)

        self.msg_box = Sprite('msg_box.png', position=(800, 900), opacity=50)
        self.add(self.msg_box, z=10000)
        self.msg_lbl = Label("Conditions: \nwell", (-170, 0), bold=True, multiline=True, width=350, font_size=16, font_name=FONTS)
        self.msg_box.add(self.msg_lbl)

        self.dollars = Sprite('dollars_b.png', position=(50, 920), scale=0.7)
        self.add(self.dollars, z=10000)

        self.fox_shtg = Sprite('fox_4.png', position=(780, 40), scale=0.3)
        self.add(self.fox_shtg)
        self.shotgun = Sprite('shotgun.png', position=(800, 50), scale=0.8)
        self.add(self.shotgun)
        self.hunting = 0

        self.died_hunger = 0
        self.died_age = 0
        self.born = 0
        self.new_eggs = 0
        self.last_time_fox_killed = 0
        self.new_fox = 0

        self.clock_day = Label("Day", (920, 60), bold=True, font_size=20, font_name=FONTS)
        self.clock_hour = Label("Hour", (920, 20), bold=True, font_size=20, font_name=FONTS)
        self.add(self.clock_day)
        self.add(self.clock_hour)
        self.clock = 0
        self._update_clock()

        #self._update(0)
        self.schedule_interval(self._update, 3)

    def _update_clock(self):
        days = self.clock / DAY_LENGTH
        hour = 8 + self.clock % DAY_LENGTH
        self.clock_day.element.text = "Day %d" % (days+1) 
        self.clock_hour.element.text = "%d:00" % hour

    def _count_animals(self):
        eggs_num = 0
        chicks_num = 0
        chickens_num = 0
        hens_num = 0
        roosters_num = 0
        for a in self.animals:
            if a.age <= EGG_AGE[1]:
                eggs_num += 1
            elif a.age <= CHICK_AGE[1]:
                chicks_num += 1
            elif a.age <= CHICKEN_AGE[1]:
                chickens_num += 1
            else:
                if a.male:
                    roosters_num += 1
                else:
                    hens_num += 1
        self.hens_cnt.set(hens_num)
        self.roosters_cnt.set(roosters_num)
        self.chicks_cnt.set(chickens_num)
        self.chickens_cnt.set(chicks_num)
        self.eggs_cnt.set(eggs_num)

    def on_mouse_press (self, x, y, buttons, modifiers):
        print x, y
        if self.feeder.is_inside(x, y):
            self.feeder.clicked()
        for i, c in enumerate([self.hens_cnt, self.roosters_cnt, self.chickens_cnt, self.eggs_cnt]):
            if is_inside(c.sprite, x, y):
                for a in self.animals:
                    found = False
                    if i == 0: # hens
                        if is_hen(a):
                            found = True
                    elif i == 1: # roosters
                        if is_rooster(a):
                            found = True
                    elif i == 2: # chickens
                        if is_chicken(a):
                            found = True
                    elif i == 3: # eggs
                        if is_egg(a):
                            found = True
                    if found:
                        soundex.play("coins%d.wav" % randint(1,4))
                        c.sell(a.get_sprite(), self.cash_cnt, self.dollars)
                        self.animals.remove(a)
                        a.destroy()
                        break
        if is_inside(self.shotgun, x, y):
            log("will be hunting")
            self.hunting = 1
            self._show_msg("There will be fox hunting\nthis night")
            
    def add_animal(self, a):
        self.add(a, z=1024 - (a.y - a.height/2))

    def _generate_animals(self, num, age_range, male):
        l = []
        for i in range(num):
            y = (768-670) + randint(80, 670-160)
            a = Animal(randint(80, 1024-80),
                       y,
                       randint(0, 1),      # direction
                       male,
                       randint(*age_range))    # age
            l.append(a)
            self.add_animal(a.get_sprite())
        return l

    def _update(self, dt):
        self.clock += 1
        log(("="*30) + "\nclock %d" % self.clock)
        self._update_clock()

        # dying from hunger
        no_food_period = self.clock - self.feeder.last_feeding_time
        log("no food period %d" % no_food_period)
        if no_food_period < DAY_LENGTH:
            ratio = 0
        elif no_food_period < DAY_LENGTH*2:
            ratio = 0.3
        elif no_food_period < DAY_LENGTH*3:
            ratio = 0.6
        else:
            ratio = 1.1
        died = 0
        for a in self.animals:
            r = random()            
            if r > 0 and r < ratio:
                log("hunger %f < %f" % (r, ratio))
                a.kill()
                self.animals.remove(a)
                died += 1
        self.died_hunger += died
        log("died from hunger %d, acc %d" % (died, self.died_hunger))

        # fox attack
        killed_by_fox = 0
        if self.clock % DAY_LENGTH == 0:
            TIME_TO_FOX = 10
            if self.new_fox < TIME_TO_FOX:
                self.new_fox += randint(0, 5)
            log("new fox in %d" % (TIME_TO_FOX - self.new_fox))
            if self.new_fox > TIME_TO_FOX and random() > 0.3: # there is new fox and it is 70% chance of attack
                for i in range(min(randint(0, 3), len(self.animals))):
                    a = self.animals[i]
                    a.kill()
                    self.animals.remove(a)
                    killed_by_fox += 1
                log("killed by fox %d" % killed_by_fox)

        # aging animals & laying eggs
        eggs = []
        died = 0
        is_rooster_present = False
        for a in self.animals: # check for rooster if eggs can be laid
            is_rooster_present = is_rooster(a)
            if is_rooster_present:
                break
        for a in self.animals:
            a.update(self.clock)
            r = random()
            if self.clock % DAY_LENGTH == 0 and is_rooster_present and is_hen(a) and r > 0.95:
                male = (int(round(random()*0.8)) == 1)
                eggs.append(self._generate_animals(1, (0, 0), male)[0])
            if a.age == OLD_AGE[1]:
                self.animals.remove(a)
                died += 1
        self.died_age += died
        log("died of age %d, acc %d" % (died, self.died_age))
        self.animals.extend(eggs)
        log("laid eggs %d" % len(eggs))
        self._count_animals()
        self.new_eggs += len(eggs)


        # preparing day message
        if self.clock % DAY_LENGTH == 0:
            died = self.died_hunger + killed_by_fox + self.died_age
            text = "Day %d. " % (self.clock / DAY_LENGTH + 1)
            if died > 0 or self.new_eggs > 0:
                text += "Last day"
                if self.new_eggs > 0:
                    text += " %d eggs was laid" % self.new_eggs
                if died > 0:
                    if self.new_eggs > 0:
                        text += " and"
                    text += " %d birds died.\n" % died
                if self.died_age > 0:
                    text += "%d were too old.\n" % self.died_age
                if self.died_hunger > 0:
                    text += "%d died from hunger.\n" % self.died_hunger
                if killed_by_fox > 0:
                    text += "%d killed by fox." % killed_by_fox
            self._show_msg(text)
            t = "".join([ "\t" + l + "\n" for l in text.split("\n") ])
            log(t)
            self.died_hunger = 0
            self.died_age = 0

        if self.hunting and self.clock % DAY_LENGTH == 0:
            self.hunting = 2
            self._hide_msg()
            director.push(FadeTransition(hunting.get_scene(), 1.0))
            

    def _hide_msg(self):
        self.msg_box.do(Place((800, 900)))        

    def _show_msg(self, text):
        self.msg_lbl.element.text = text
        m = MoveBy((0, -200), 2)
        a = Accelerate(m, 0.6)
        a = a + Delay(4) + Reverse(a)
        self.msg_box.do(a)

class GameScene(cocos.scene.Scene):
    def __init__(self, farm):
        super(GameScene, self).__init__()
        self.farm = farm

    def on_enter(self):
        super(GameScene, self).on_enter()
        soundex.set_music('chickens.mp3')
        soundex.play_music()
        log("entering game scene")
        if self.farm.hunting == 2:
            self.farm.hunting = 0
            self.farm.new_fox = 0

    def on_exit(self):
        super(GameScene,self).on_exit()
        soundex.stop_music()

def get_game_scene():
    farm = CtrlLayer()
    s = GameScene(farm)
    s.add(BackgroundLayer(), z=0)
    s.add(farm, z=1)
    return s
