from cocos.director import director
from cocos.layer import Layer, ColorLayer, MultiplexLayer
from cocos.scene import Scene
from cocos.scenes.transitions import FadeTransition
#from cocos.actions import *
from cocos.sprite import Sprite
from cocos.menu import Menu, MenuItem, ToggleMenuItem, MultipleMenuItem, shake, shake_back
from cocos.text import Label

import pyglet
from pyglet import gl, font
from pyglet.window import key

import game_scene

import soundex
from constants import *

class TitleLayer( Layer ):
    def __init__(self):
        super( TitleLayer, self ).__init__()

        w,h = director.get_window_size()
        self.font_title = {}

        # you can override the font that will be used for the title and the items
        self.font_title['font_name'] =  FONTS # 'Edit Undo Line BRK'
        self.font_title['font_size'] = 26
#        self.font_title['color'] = (204,164,164,255)
        self.font_title['color'] = (255,204,204,255)
        self.font_title['anchor_y'] ='top'
        self.font_title['anchor_x'] ='right'
        #title = Label('A LOVE STORY', **self.font_title )
        #title.position=(w-10,30)
        #self.add(title,z=1)

        self.font_title['color'] = (64,32,32,255)
        #title = Label('A LOVE STORY', **self.font_title )
        #title.position=(w-12,32)
        #self.add(title,z=0)
        bg = Sprite("chickencoop.png", position=(1024/2, 768/2))
        self.add(bg)

class ScoresLayer( ColorLayer):

    FONT_SIZE=22

    is_event_handler = True #: enable pyglet's events

    def __init__(self):

        w,h = director.get_window_size()
        super( ScoresLayer,self).__init__( 32,32,32,16, width=w, height=h-97)

        self.font_title = {}

        # you can override the font that will be used for the title and the items
        self.font_title['font_name'] = FONTS # 'Edit Undo Line BRK'
        self.font_title['font_size'] = 72
#        self.font_title['color'] = (204,164,164,255)
        self.font_title['color'] = (255,255,255,255)
        self.font_title['anchor_y'] ='top'
        self.font_title['anchor_x'] ='center'

        title = Label('Chikencoop', **self.font_title )

        title.position=(w/2.0,h-5)

        self.add(title,z=1)

        self.table = None

    def on_enter( self ):
        super(ScoresLayer,self).on_enter()

        scores = hiscore.hiscore.get()

        if self.table:
            self.remove_old()

        self.table =[]
        for idx,s in enumerate(scores):

            pos= Label( '%d:' % (idx+1), font_name='Edit Undo Line BRK',
                        font_size=self.FONT_SIZE,
                        anchor_y='top',
                        anchor_x='left',
                        color=(255,255,255,255) )

            name = Label( s[1], font_name='Edit Undo Line BRK',
                        font_size=self.FONT_SIZE,
                        anchor_y='top',
                        anchor_x='left',
                        color=(255,255,255,255) )

            score = Label( str(s[0]), font_name='Edit Undo Line BRK',
                        font_size=self.FONT_SIZE,
                        anchor_y='top',
                        anchor_x='right',
                        color=(255,255,255,255) )

            lvl = Label( str(s[2]), font_name='Edit Undo Line BRK',
                        font_size=self.FONT_SIZE,
                        anchor_y='top',
                        anchor_x='right',
                        color=(255,255,255,255) )

            self.table.append( (pos,name,score,lvl) )

        self.process_table()

    def remove_old( self ):
        for item in self.table:
            pos,name,score,lvl = item
            self.remove(pos)
            self.remove(name)
            self.remove(score)
            self.remove(lvl)
        self.table = None

    def process_table( self ):
        w,h = director.get_window_size()

        for idx,item in enumerate(self.table):
            pos,name,score,lvl = item

            posy = h - 100 - ( (self.FONT_SIZE+15) * idx )

            pos.position=( 5, posy)
            name.position=( 48, posy)
            score.position=( w-150, posy )
            lvl.position=( w-10, posy)

            self.add( pos, z=2 )
            self.add( name, z=2 )
            self.add( score, z=2 )
            self.add( lvl, z=2 )

    def on_key_press( self, k, m ):
        if k in (key.ENTER, key.ESCAPE, key.SPACE):
            self.parent.switch_to( 0 )
            return True

    def on_mouse_release( self, x, y, b, m ):
        self.parent.switch_to( 0 )
        return True


class OptionsMenu( Menu ):
    def __init__(self):
        super( OptionsMenu, self).__init__('Chikencoop') 
        self.select_sound = soundex.load('move.mp3')

        # you can override the font that will be used for the title and the items
        self.font_title['font_name'] = FONTS #'Edit Undo Line BRK'
        self.font_title['font_size'] = 72
#        self.font_title['color'] = (204,164,164,255)
        self.font_title['color'] = FONT_COLOR # (255,255,255,255)


        self.font_item['font_name'] = FONTS #'Edit Undo Line BRK',
        self.font_item['color'] = FONT_COLOR #(32,16,32,255)
        self.font_item['font_size'] = 32
        self.font_item_selected['font_name'] = FONTS #'Edit Undo Line BRK'
        self.font_item_selected['color'] = FONT_COLOR #(32,16,32,255)
        self.font_item_selected['font_size'] = 46

        # you can also override the font size and the colors. see menu.py for
        # more info

        # example: menus can be vertical aligned and horizontal aligned
        self.menu_anchor_y = font.Text.CENTER
        self.menu_anchor_x = font.Text.CENTER

        items = []

        self.volumes = ['Mute','10','20','30','40','50','60','70','80','90','100']

        #self.levels = [ str(i) for i in range( len(levels.levels))]
        self.levels = ["1", "2", "3"]

#        items.append( MultipleMenuItem(
#                        'Start Level: ', 
#                        self.on_level,
#                        self.levels,
#                        1 )#int(state.start_level) )
#                    )

        items.append( MultipleMenuItem(
                        'SFX volume: ', 
                        self.on_sfx_volume,
                        self.volumes,
                        int(soundex.sound_vol * 10) )
                    )
        items.append( MultipleMenuItem(
                        'Music volume: ', 
                        self.on_music_volume,
                        self.volumes,
                        int(soundex.music_player.volume * 10) )
                    )
        items.append( ToggleMenuItem('Show FPS:', self.on_show_fps, director.show_FPS) )
        #items.append( MenuItem('Fullscreen', self.on_fullscreen) )
        items.append( MenuItem('Back', self.on_quit) )
        self.create_menu( items, shake(), shake_back() )

    def on_fullscreen( self ):
        director.window.set_fullscreen( not director.window.fullscreen )

    def on_quit( self ):
        self.parent.switch_to( 0 )

    def on_show_fps( self, value ):
        director.show_FPS = value

    def on_sfx_volume( self, idx ):
        vol = idx / 10.0
        soundex.sound_volume( vol )

    def on_level( self, idx ):
        state.start_level = idx

    def on_music_volume( self, idx ):
        vol = idx / 10.0
        soundex.music_volume( vol )

class MainMenu( Menu ):

    def __init__(self):
        super( MainMenu, self).__init__('Chikencoop') 

        self.select_sound = soundex.load('move.mp3')
        self.activate_sound = soundex.load('rooster.wav')

        # you can override the font that will be used for the title and the items
        # you can also override the font size and the colors. see menu.py for
        # more info
        self.font_title['font_name'] = FONTS
        self.font_title['font_size'] = 72
#        self.font_title['color'] = (204,164,164,255)
        self.font_title['color'] = FONT_COLOR #(255,255,255,255)

        self.font_item['font_name'] = FONTS
        self.font_item['color'] = FONT_COLOR #(32,16,32,255)
        self.font_item['font_size'] = 32
        self.font_item_selected['font_name'] = FONTS
        self.font_item_selected['color'] = FONT_COLOR #(32,16,32,255)
        self.font_item_selected['font_size'] = 46


        # example: menus can be vertical aligned and horizontal aligned
        self.menu_anchor_y = font.Text.CENTER
        self.menu_anchor_x = font.Text.CENTER

        items = []

        items.append( MenuItem('New Game', self.on_new_game) )
        items.append( MenuItem('Options', self.on_options) )
        #items.append( MenuItem('Scores', self.on_scores) )
        items.append( MenuItem('Quit', self.on_quit) )

        self.create_menu( items, shake(), shake_back() )

    def on_new_game(self):
        #import hunting
        #director.push( FadeTransition(
        #    hunting.get_scene(), 1.0 ) )
        director.push( FadeTransition(
            game_scene.get_game_scene(), 1.0 ) )

    def on_options( self ):
        self.parent.switch_to(1)


    def on_scores( self ):
        self.parent.switch_to(2)

    def on_quit(self):
        pyglet.app.exit()

class MenuControlLayer( Layer ):

    
    def __init__(self, model):
        super(MenuControlLayer,self).__init__()
        self.model = model

        self.schedule_interval(self.step, 0.5)

#        self.angle = random.randrange(360)
        self.angle = 70

    def step(self, dt):
    
        self.angle = self.angle + random.randrange(-30,0)
        self.angle = self.angle % 360

        force_value = 100
        r = math.radians(self.angle)
        force = (  math.cos(r) * force_value, math.sin(r) * force_value )
        self.model.chain[0].apply_impulse(force, (0,0) )

def get_menu_scene():

    scene = Scene()
    scene.add( MultiplexLayer(
                    MainMenu(), 
                    OptionsMenu()#,
                    #ScoresLayer(),
                    ),
                z=2 ) 
    scene.add( TitleLayer(), z=1)
    #game_model = GameLayer(demo=True)
    #scene.add( game_model, z=1)
    #scene.add( MenuControlLayer( game_model), z=0)

    return scene
